"""
open-fdd/                # Repository root
├── open_fdd/            # Python package root
│   ├── __init__.py
│   ├── air_handling_unit/
│   │   ├── __init__.py
│   │   ├── faults/
│   │   │   ├── __init__.py
│   │   │   ├── helper_utils.pya
│   │   │   ├── fault_condition.py
│   │   │   ├── fault_condition_one.py
│   │   │   ├── fault_condition_two.py
│   │   │   └── ... # other fault conditions
│   │   ├── examples/
│   │   ├── reports/
│   │   │   ├── __init__.py
│   │   │   ├── base_report.py
│   │   │   ├── report_fc1.py
│   │   │   ├── report_fc2.py
│   │   │   └── ... # other reports
│   │   └── images/
│   │       ├── fc1_definition.png
│   │       ├── fc2_definition.png
│   │       └── ... # other images
│   ├── black_box/
│   │   └── ... # subject to change being experimental
├── tests/
│   ├── __init__.py
│   ├── air_handling_unit/
│   │   ├── __init__.py
│   │   ├── test_ahu_fc1.py
│   │   ├── test_ahu_fc2.py
│   │   └── ... # other tests
├── setup.py
├── README.md
├── LICENSE
└── requirements.txt

"""
